CompiledCode subclass:#JavaMethod
	instanceVariableNames:'javaByteCode numArgs numLocals returnsVoid accessFlags name
		signature exceptionTable lineNumberTable javaClass numStack'
	classVariableNames:'SignatureTypeCodes'
	poolDictionaries:''
	category:'Java-Classes'
!


!JavaMethod class methodsFor:'initialization'!

initialize
    SignatureTypeCodes := IdentityDictionary new.
    SignatureTypeCodes at:$B put:#byte.
    SignatureTypeCodes at:$C put:#char.
    SignatureTypeCodes at:$D put:#double.
    SignatureTypeCodes at:$F put:#float.
    SignatureTypeCodes at:$I put:#int.
    SignatureTypeCodes at:$J put:#long.
    SignatureTypeCodes at:$S put:#'unsigned short'.
    SignatureTypeCodes at:$Z put:#boolean.
    SignatureTypeCodes at:$L put:#object.
    SignatureTypeCodes at:$[ put:#array.

    "
     JavaMethod initialize
    "
! !

!JavaMethod class methodsFor:'signature parsing'!

argSigArrayFromSignature:aSignature
    "given a signature, return a specArray for the arguments"

    |s argSpec retvalSpec|

    s := aSignature readStream.
    s next ~~ $( ifTrue:[self halt. ^ name].

    ^ self argSigArrayFromStream:s.

    "
     JavaMethod argSigArrayFromSignature:'(LObject;)V'
     JavaMethod argSigArrayFromSignature:'(BB)S'      
     JavaMethod argSigArrayFromSignature:'(LObject;LObject;II)V'      
    "
!

argSigArrayFromStream:s
    "parse an argSpec, return an array of specs - see java doc"

    |argSpec spec|

    spec := #().
    [s atEnd or:[s peek == $)]] whileFalse:[
"/        s peek == Character space ifTrue:[
"/            s next
"/        ] ifFalse:[
            argSpec := self fieldTypeFromStream:s.
            spec := spec copyWith:argSpec.
"/        ]
    ].
    ^ spec


!

argSpecFromStream:s
    "parse an argSpec - see java doc"

    |argSpec spec|

    spec := ''.
    [s atEnd or:[s peek == $)]] whileFalse:[
"/        s peek == Character space ifTrue:[
"/            s next
"/        ] ifFalse:[
            argSpec := self fieldTypeFromStream:s.
            spec size ~~ 0 ifTrue:[
                spec := spec , ' '
            ].
            spec := spec , argSpec.
"/        ]
    ].
    ^ spec


!

fieldTypeFromStream:s
    "parse a fieldTypeSpec - see java doc"

    |typeChar typeSym spec elType size className|

    typeChar := s next.

    typeSym := SignatureTypeCodes at:typeChar ifAbsent:#unknown.

    typeSym == #unknown ifTrue:[
        ^ typeSym
    ].
    typeSym == #object ifTrue:[
        className := s upTo:$;.
        "/ strip off default
        (className startsWith:'java/lang/') ifTrue:[
            ^ className copyFrom:11
        ].
        ^ className
    ].
    typeSym == #array ifTrue:[
        s peek isDigit ifTrue:[
            size := Integer readFrom:s.
            elType := self fieldTypeFromStream:s.
            ^ elType , '[' , size printString , ']'
        ].
        elType := self fieldTypeFromStream:s.
        ^ elType , '[]'
    ].
    ^ typeSym
!

numArgsFromSignature:aSignature
    "given a signature, return the number of args"

    |s argSpec|

    s := aSignature readStream.
    s next ~~ $( ifTrue:[self halt. ^ name].

    ^ self numArgsFromStream:s.

    "
     JavaMethod numArgsFromSignature:'(LObject;)V'
     JavaMethod numArgsFromSignature:'(BB)S'      
     JavaMethod numArgsFromSignature:'()V'      
    "
!

numArgsFromStream:s
    "parse an argSpec - see java doc"

    |argSpec n|

    n := 0.
    [s atEnd or:[s peek == $)]] whileFalse:[
        self fieldTypeFromStream:s.
        n := n + 1.
    ].
    ^ n
!

retValSpecFromSignature:aSignature
    "given a signature, return a spec"

    |s argSpec retvalSpec|

    s := aSignature readStream.
    s next ~~ $( ifTrue:[self halt. ^ name].

    argSpec := self argSpecFromStream:s.

    s next ~~ $) ifTrue:[self halt. ^ name].

    retvalSpec := self retvalSpecFromStream:s.

    ^ retvalSpec 

    "
     JavaMethod retValSpecFromSignature:'(LObject;)V'
     JavaMethod retValSpecFromSignature:'(BB)S'      
    "
!

returnsVoidFromSignature:aSignature
    "given a signature, return true if it returns void, false if not"

    |s argSpec retvalSpec|

    s := aSignature readStream.
    s next ~~ $( ifTrue:[self halt. ^ true].

    self argSpecFromStream:s.

    s next ~~ $) ifTrue:[self halt. ^ true].

    ^ (self retvalSpecFromStream:s) = 'void'

    "
     JavaMethod returnsVoidFromSignature:'(LObject;)V' 
     JavaMethod returnsVoidFromSignature:'(BB)S'       
    "
!

retvalSpecFromStream:s
    "parse a retvalSpec - see java doc"

    |argSpec spec|

    s atEnd ifTrue:[self halt. ^ 'void'].
    s peek == $V ifTrue:[^ 'void'].
    ^ self fieldTypeFromStream:s

!

specFromSignature:aSignature withName:name
    "given a signature, return a spec"

    |s argSpec retvalSpec|

    s := aSignature readStream.
    s next ~~ $( ifTrue:[self halt. ^ name].

    argSpec := self argSpecFromStream:s.

    s next ~~ $) ifTrue:[self halt. ^ name].

    retvalSpec := self retvalSpecFromStream:s.

    ^ retvalSpec , ' ' , name , ' (' , argSpec , ')'

    "
     JavaMethod specFromSignature:'(LObject;)V' withName:'foo' 
     JavaMethod specFromSignature:'(BB)S'       withName:'foo' 
    "
! !

!JavaMethod methodsFor:'accessing'!

argSignature
    ^ self class argSigArrayFromSignature:signature
!

constantPool
    ^ javaClass constantPool

    "Modified: 16.4.1996 / 12:36:27 / cg"
    "Created: 16.4.1996 / 15:28:50 / cg"
!

javaByteCode
    ^ javaByteCode

    "Modified: 16.4.1996 / 12:36:27 / cg"
    "Created: 16.4.1996 / 14:55:44 / cg"
!

javaClass
    ^ javaClass

    "Modified: 16.4.1996 / 12:36:27 / cg"
    "Created: 16.4.1996 / 14:55:44 / cg"
!

methodArgAndVarNames
    ^ ((1 to:self numArgs) collect:[:i | 'arg' , i printString])
      ,
      ((1 to:self numVars) collect:[:i | 'local' , i printString])

!

methodArgNames
    ^ (1 to:self numArgs) collect:[:i | 'arg' , i printString]
!

name
    ^ name

    "Created: 16.4.1996 / 11:34:22 / cg"
!

numArgs
    numArgs isNil ifTrue:[^ 0].
    ^ numArgs
!

numVars
    numLocals isNil ifTrue:[^ 0].
    ^ numLocals
!

numberOfMethodArgs:n
    numArgs := n
!

retValSignature
    ^ self class retValSpecFromSignature:signature

!

returnsVoid
    ^ returnsVoid
!

setAccessFlags:flags
    accessFlags := flags.

    "Created: 16.4.1996 / 11:34:14 / cg"
!

setCode:codeBytes maxStack:max_stack maxLocals:max_locals u1:unknown1 u2:unknown2
    javaByteCode := codeBytes.
    numStack := max_stack.
    numLocals := max_locals.

    self displayString printNL.
    '   nStack: ' print. numStack print. 
    ' nLocal: ' print. numLocals print. 
    ' u1: ' print. unknown1 print.
    ' u2: ' print. unknown2 printNL.

!

setExceptionTable:anArray
    exceptionTable := anArray.

    "Created: 16.4.1996 / 12:34:04 / cg"
!

setJavaClass:aJavaClass
    javaClass := aJavaClass

    "Modified: 16.4.1996 / 12:36:27 / cg"
    "Created: 16.4.1996 / 15:28:15 / cg"
!

setLineNumberTable:anArray
    lineNumberTable := anArray.

    "Created: 16.4.1996 / 12:34:04 / cg"
    "Modified: 16.4.1996 / 12:49:06 / cg"
!

setMaxStack:max_stack maxLocals:max_locals u1:unknown1 u2:unknown2
    numStack := max_stack.
    numLocals := max_locals.
!

setName:aString
    name := aString asSymbol.

    "Created: 16.4.1996 / 11:34:22 / cg"
!

setSignature:aString
    signature := aString asSymbol.

    self numberOfMethodArgs:(self class numArgsFromSignature:aString).
    returnsVoid := self class returnsVoidFromSignature:aString.

    "Created: 16.4.1996 / 11:34:29 / cg"
!

signature
    ^ signature 

    "Created: 16.4.1996 / 11:34:29 / cg"
!

source
    |s|

    s := '' writeStream.
    self decompileTo:s.
    ^ s contents
!

sourceFilename
    ^ javaClass sourceFile
! !

!JavaMethod methodsFor:'decompiling'!

decompileTo:aStream
    JavaDecompiler decompile:self to:aStream.
    ^ true


! !

!JavaMethod methodsFor:'printing & storing'!

displayString
    ^ 'JavaMethod(' , javaClass name , '::' , self signatureName , ')'
!

printStringForBrowserWithSelector:selector
    ^ self signatureName
!

signatureName
    "return a string to be used when browsing"

    ^ self class specFromSignature:signature withName:name
! !

!JavaMethod methodsFor:'queries'!

hasHandlerFor:anException
    exceptionTable isNil ifTrue:[^ false].
    self halt.
    ^ false
!

isAbstract
    ^ (accessFlags bitAnd:16r0400) ~~ 0
!

isFinal
    ^ (accessFlags bitAnd:16r0010) ~~ 0
!

isNative
    ^ (accessFlags bitAnd:16r0100) ~~ 0
!

isPrivate
    ^ (accessFlags bitAnd:16r0002) ~~ 0

!

isProtected
    ^ (accessFlags bitAnd:16r0004) ~~ 0
!

isPublic
    ^ (accessFlags bitAnd:16r0001) ~~ 0

!

isStatic
    ^ (accessFlags bitAnd:16r0008) ~~ 0
!

isSynchronized
    ^ (accessFlags bitAnd:16r0020) ~~ 0
!

isUnloaded
    ^ false
!

lineNumberForPC:pc
    |num text|

    num := nil.

    lineNumberTable isNil ifTrue:[
        "/ decompile and look which line the pc falls into
        text := self source asCollectionOfLines.

        text keysAndValuesDo:[:lineNr :line |
            |nr|

            nr := Integer readFrom:line onError:0.
            nr >= pc ifTrue:[
                ^ lineNr
            ]
        ].
        ^ num
    ].
    self halt.
    ^ num
!

who
    "return the class and selector of where I am defined in."

    |idx|

    idx := javaClass methodArray identityIndexOf:self.
    idx == 0 ifTrue:[^ nil].
    ^ Array with:javaClass with:(javaClass selectorArray at:idx)
! !

!JavaMethod class methodsFor:'documentation'!

version
    ^ '$Header: /home/jv/Projects/SmalltalkX/repositories/cvs/stx/libjava/JavaMethod.st,v 1.9 1996/05/02 15:18:31 cg Exp $'
! !
JavaMethod initialize!
